%  Description:
%   
%   Find miniumum distance from spots to surface object and add this as a
%   statistic to the spots. This distance is not limited by the bit depth
%   of the data (i.e. can be greater than 255 for 8 bit data)
% 
%

function aImarisApplication = spotsMinDistanceToSurface(aImarisApplicationID)
% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  imarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  imarisApplication = aImarisApplicationID;
end
vImarisDataSet = imarisApplication.GetDataSet;
 


%use the bottom set of spots and surfaces for now
spotsAndSurfs =  xtgetsporfaces(imarisApplication,'Both');
for i = 1:size(spotsAndSurfs,2)
   object = spotsAndSurfs(i).ImarisObject;
   if imarisApplication.GetFactory.IsSurfaces(object)
       surfaces = imarisApplication.GetFactory.ToSurfaces(object);
       surfaceName = spotsAndSurfs(i).Name;
   else
       spots = imarisApplication.GetFactory.ToSpots(object);
   end
end



vDataMin = [vImarisDataSet.GetExtendMinX, vImarisDataSet.GetExtendMinY, vImarisDataSet.GetExtendMinZ];
vDataMax = [vImarisDataSet.GetExtendMaxX, vImarisDataSet.GetExtendMaxY, vImarisDataSet.GetExtendMaxZ];
vDataSize = [vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, vImarisDataSet.GetSizeZ];

vProgressDisplay = waitbar(0, 'Calculating distances');

% preallocate mask array for speed
mask = zeros(vDataSize);

pixelSizeZ = (vDataMax(3) - vDataMin(3)) / vDataSize(3);
pixelSizeXY = (vDataMax(2) - vDataMin(2)) / vDataSize(2);
numTimePoints = vImarisDataSet.GetSizeT;
spotTimeIndices = spots.GetIndicesT;
spotXYZ = spots.GetPositionsXYZ;
numSpots = length(spotTimeIndices);
distances = zeros(numSpots,1);

for timeIndex = 0:numTimePoints-1;
%     Get the mask DataSet, 8 bit binary data set
    vMaskDataSet = surfaces.GetMask(vDataMin(1), vDataMin(2), vDataMin(3), ...
        vDataMax(1), vDataMax(2), vDataMax(3), vDataSize(1), vDataSize(2), vDataSize(3), timeIndex);
    for vIndexZ = 1:vDataSize(3)
        vSlice = vMaskDataSet.GetDataSliceBytes(vIndexZ-1, 0, 0);
%         1's are within surface, zeros outside
        
        mask(:,:,vIndexZ) = vSlice;
    end
%     use mask data to calculate distance map using bwdistsc.m
    distanceTransform = bwdistsc(mask,[pixelSizeXY pixelSizeXY pixelSizeZ]);
    
%     iterate through each spot at this timepoint, calculate its distance
%     from the surface based on the distanceTransform, and add that as a
%     custom statistic to imaris
    startIndex = find(spotTimeIndices == timeIndex,1,'first');
    endIndex = find(spotTimeIndices == timeIndex,1,'last');
    
    
    for spotIndex = startIndex:endIndex
        coordinates = spotXYZ(spotIndex,:);
        indices = floor((coordinates - vDataMin) ./ [pixelSizeXY pixelSizeXY pixelSizeZ]);
        distances(spotIndex) = distanceTransform(indices(1),indices(2),...
            max(1,indices(3)));
    end
    waitbar((timeIndex+1)/numTimePoints, vProgressDisplay);  
end
close(vProgressDisplay);


% Code fore creating a custom statistic
% Create the distance statistic name list.

%add exsiting coretex distance data rather than calulating it
% surfaceName = 'surface';
% temp = load('E:\Data\7-30 GFP CD8 CD11c mcherry SL8 at 2 hr\whole LN starting all spots distances','dist');
% distances = temp.dist;
% numSpots = length(distances);
% spotTimeIndices = zeros(numSpots,1);

distBaseName = ['Distance to ' surfaceName ' ']; % Trailing space is to get Imaris to behave and show the units.
distNames = repmat({distBaseName}, [numSpots 1]); 

% Create the stat unit list.
imarisUnits = char(imarisApplication.GetDataSet.GetUnit);
if isempty(imarisUnits)
    imarisUnits = 'um';
end % if
distUnits = repmat({imarisUnits}, [numSpots, 1]); 

% Assemble the factors cell array.
distFactors = cell(3, numSpots);

% Set the Category.
distFactors(1, :) = repmat({'Spot'}, [numSpots 1]);

% Set the Collection to an empty string.
distFactors(2, :) = repmat({''}, [numSpots 1]);

% Set the Time.
distFactors(3, :) = num2cell(spotTimeIndices + 1);

% Convert the time points to strings.
distFactors(3, :) = cellfun(@num2str, distFactors(3, :), 'UniformOutput', 0);

% Create the factor names.
factorNames = {'Category', 'Collection', 'Time'};

% Create the stat IDs.
distIDs = transpose(0:numSpots - 1);

% Send the distance statistics to Imaris.
spots.AddStatistics(distNames, distances, distUnits, ...
    distFactors, factorNames, distIDs)

end
